<b>{{title}}</b>

<table width="100%">
{% for row in table %}
  <tr>
    <td>{{row.field_attributes.name}}</td>
    <td>{{row.value|to_html(row.field_attributes.delegate)}}</td>
  </tr>
{% endfor %}
</table>
